#ifndef oper_H
#define oper_H

#include <stdexcept>
#include "hypercube.h"
#include "floatHyper.h"

using namespace SEP;

// base class for linear operators
class oper {
protected:
    std::shared_ptr<hypercube> _domain;
    std::shared_ptr<hypercube> _range;

public:
    // default constructor
	oper(){}

    // virtual destructor
    virtual ~oper(){}

    // constructor
    oper(const std::shared_ptr<hypercube> domain, const std::shared_ptr<hypercube> range){
        _domain = domain->clone();
        _range = range->clone();
    }

    // abstract method to clone the operator
    //virtual oper * clone() const = 0;

    // get domain
    std::shared_ptr<hypercube> getDomain() const {return _domain;}

    // get range
    std::shared_ptr<hypercube> getRange() const {return _range;}

    // set domain and range from hypercube
    void setDomainRange(const std::shared_ptr<hypercube> domain, const std::shared_ptr<hypercube> range){
        _domain = domain->clone();
        _range = range->clone();
    }

    // set domain and range from floatHyper
    void setDomainRange(const std::shared_ptr<floatHyper> mod, const std::shared_ptr<floatHyper> dat){
        _domain = mod->getHyper()->clone();
        _range = dat->getHyper()->clone();
    }

    // check domain and range with hypercube
    inline void checkDomainRange(const std::shared_ptr<hypercube> domain, const std::shared_ptr<hypercube> range) const {
        _domain->checkSame(domain);
        _range->checkSame(range);
    }

    // check domain and range with float2DReg
    inline void checkDomainRange(const std::shared_ptr<floatHyper> mod, const std::shared_ptr<floatHyper> dat) const {
        _domain->checkSame(mod->getHyper());
        _range->checkSame(dat->getHyper());
    }

};

#endif