#ifndef cgls_H
#define cgls_H

#include "float2DReg.h"
#include "oper.h"
#include "oper1D.h"
#include "oper2D.h"

using namespace SEP;

// Linear conjugate gradient least squared CGLS solver of min{norm2(Lm - d)}
// according to Aster (Parameter Estimation and Inverse Problems), chap 6
class cgls{
protected:
    unsigned int _niter;
    float _threshold; // stopper based on rate of convergence
    std::vector <float> _func;

public:
    // default constructor
    cgls(){}

    // constructor
    cgls(unsigned int niter, float threshold=0){
        _niter = niter;
        _threshold = threshold;
        _func = {};
    }

    // set the number of iterations
    void setNiter(unsigned int niter) {_niter = niter;}

    // set the threshold
    void setThreshold(float threshold){_threshold = threshold;}

    // run the solver for 1D problem
    void run(const oper1D * op, std::shared_ptr<float1DReg> mod, const std::shared_ptr<float1DReg> dat);

    // run the solver for 2D problem
    void run(const oper2D * op, std::shared_ptr<float2DReg> mod, const std::shared_ptr<float2DReg> dat);

    // test the solver for a full rank system Lm = d with 1D vectors
    void test();

    // reset the objective function
    void resetFunc(){
        _func.clear();
    }

    // plot objective function
    void plotFunc(bool logscale = false);

    // save objective function to png format
    void saveFunc(bool logscale = false, std::string name = "objectiveFunc", std::string directory = "./");
};

#endif