#ifndef functions_H
#define functions_H

#include <iostream>
#include <stdio.h>
#include <cmath>
#include <math.h>
#include <fstream>
#include <string>
#include <cstring>
#include <vector>
#include <array>
#include <stdlib.h>
#include <cstdlib>
#include <ctime>
#include <complex>
#include "Trace.h"

using namespace std;


// Functions declaration

// Print the values of a vector
void print(vector <TYPE> &v,string message="", ostream &o=cout);

// Print the values of an array
template <size_t N>
void print(array <TYPE,N> &a,string message="", ostream &o=cout){
	string espace(message.size(), ' ');
	o << message << " [" ;
	for (int j =0; j<N; j++)
		o << " " << a[j];
	o << " ]" << endl;
}

// Print a complex number
void print(complex <TYPE> & z,ostream &o=cout);

// Plot a trace from ascii file using GNUPLOT
void plot(string file, string param1="", string param2="with linespoints");

// Save the plot of a trace from ascii to png file  using GNUPLOT
void saveplot(string file, string out="", string term="png", string param1="", string param2="with linespoints");

// Output discrete Fourier transform of data vector
vector <complex<TYPE> >* DFT(vector<TYPE> * vec);

// Output discrete Fourier transform of data vector using FFTW
vector <complex<TYPE> >* FFT(vector<TYPE> * vec);

// Inverse Fourier transform
vector<TYPE>* inv_DFT(vector <complex<TYPE> >* dft);

// Inverse Fourier transform using FFTW3
vector<TYPE>* inv_FFT(vector <complex<TYPE> >* fft);

// Save real part of DFT into Trace
Trace* real(vector <complex<TYPE> >* dft, bool full_transform, TYPE SI=1, int t=0);

// Save imaginary part of DFT into Trace
Trace* imag(vector <complex<TYPE> >* dft, bool full_transform, TYPE SI=1, int t=0);

// Save modulus squared (amplitude spectrum or magnitude squared) of DFT into Trace
Trace* amp(vector <complex<TYPE> >* dft, bool full_transform, TYPE SI=1, int t=0);

// Save argument (phase angle) of DFT into Trace
Trace* phase(vector <complex<TYPE> >* dft, bool full_transform, TYPE SI=1, int t=0);

// Define the 'sinc' function
TYPE sinc(const TYPE x);

// Build a cosine windowed sinc filter
// wc is the cutoff angular frequency (percentage of PI)
// N is the half length of the filter, in number of samples
// alpha is the coefficient used for cosine windowing
Trace* sinc_filter(TYPE wc, int N=13, TYPE alpha=0.5);

// Build a centered Gaussian wavelet
// sig is the std deviation in number of samples
// N is the half length of the wavelet, in number of samples
Trace* gaussian(TYPE sig=10, int N=100);

// Build a centered Ricker wavelet
// sig is the std deviation in number of samples
// N is the half length of the wavelet, in number of samples
// Central freq is wc = sqrt(2)/(sig*si) where si is the sampling interval
Trace* ricker(TYPE sig=10, int N=100);

// Save complex vector to a 2 columns ascii file (real and imag)
void savec(vector<complex<TYPE> > * vec, string file);

// Plot a complex vector from ascii file to complex plane using GNUPLOT
void plotc(string file, string param1="", string param2="with lines");

// Save the plot pf a complex vector from ascii file to complex plane using GNUPLOT
void saveplotc(string file, string out="", string term="png", string param1="", string param2="with lines");

#endif