#ifndef pef_1d_H
#define pef_1d_H

#include <vector>

#define TYPE double

using namespace std;

// Estimate the 1d stationary Predictionary Error Filter PEF from 1D data
// The data has Nd+1 elements
// The PEF has Nf+1 elements (including the leading 1)
// The residuals can be written as r = d - Gm where
// d = (data 0)' Nd+Nf+1 vector
// m = (f1,...,fNf)' the unknown PEF coefficients
// G is the (Nd+Nf+1)xNf convolution matrix depending solely on data


// Compute the action of G on a given vector v of size=Nf
vector <TYPE>* G(vector <TYPE>* data, vector <TYPE>* v);

// Compute the action of GT (transpose G) on a given vector v of size Nd+Nf+1
vector <TYPE>* GT(vector <TYPE>* data, vector <TYPE>* v);

// Perform the dot product test for G
void G_dot_product();

// Output the L2 norm squared of a vector
TYPE norm2(vector <TYPE>* v);

// Solve the LSQ problem minimizing r using conjugate gradient method
vector <TYPE>* pef_1d(vector <TYPE>* data, unsigned int Nf, TYPE tolerance=0);


// Estimate LSQ model for Pm = d where P is a PEF
// The data has Nd+1 elements
// The PEF has Nf+1 elements (including the leading 1)
// The residuals can be written as r = d - Pm where
// d = (data 0)' Nd+Nf+1 vector
// m = (m0,...,mNd)' the unknown model coefficients
// P is the (Nd+Nf+1)x(Nd+1) convolution matrix depending solely on the PEF


// Compute the action of P on a given vector v of size=Nd
vector <TYPE>* P(vector <TYPE>* pef, vector <TYPE>* v);

// Compute the action of PT (transpose P) on a given vector v of size Nd+Nf-1
vector <TYPE>* PT(vector <TYPE>* pef, vector <TYPE>* v);

// Perform the dot product test for P
void P_dot_product();

// Solve the LSQ problem minimizing r using conjugate gradient method
// This is a deconvolution solver by CGLS
vector <TYPE>* lsq_deconvolution(vector <TYPE>* pef, vector <TYPE>* data, TYPE tolerance=0);

#endif