#include <time.h>
#include <chrono>

#include <sys/stat.h>
#include <fstream>

#include <stdio.h>
#include <stdlib.h>
#include <string>
#include <math.h>

#include "floatHyperExt.h"
#include "float2DReg.h"
#include "functions.h"
#include "fkTransform.h"
#include "complexHyperExt.h"
#include "complex2DReg.h"


using namespace SEP;


// ========================================
// ================ Main ==================
// ========================================


int main(int argc, char **argv) {
    
    std::clog << "Starting program fk_spectrum \n";
	srand ( time(0) ); 
	auto start = std::chrono::high_resolution_clock::now();

// read in the input data
    std::shared_ptr<float2DReg> dat (new float2DReg(VECEXT::readHyper(argv[1])));
    VECEXT::read(dat, argv[1]);

// read in the padding size
    int pad_n1, pad_n2;
    MB::readParam(argc, argv, "pad_n1", pad_n1, 0);
    MB::readParam(argc, argv, "pad_n2", pad_n2, 0);

// read in the output file name
    std::string output;
    MB::readParam(argc, argv, "output", output, "fk_spectrum");

// compute FK spectrum of the data
    dat = VECEXT::pad(dat, 0, pad_n2, 0, pad_n1);
    axis X = dat->getHyper()->getAxis(2);
    axis Z = dat->getHyper()->getAxis(1);
    fkTransform fk(dat->getHyper());
    X.d = fk.dk();
    X.o = fk.ok();
    Z.d = fk.df();
    Z.o = 0;
    Z.n = Z.n /2 +1;
    std::shared_ptr<complex2DReg> vec (new complex2DReg(Z, X));
    fk.forward(false,dat,vec);
    std::shared_ptr<float2DReg> datfk (new float2DReg(vec->getHyper()));
    datfk = CVECEXT::module2(vec);

// write the data fk spectrum to disk
    VECEXT::write(datfk, output);
	
	auto stop = std::chrono::high_resolution_clock::now();
	auto duration = std::chrono::duration_cast<std::chrono::microseconds>(stop - start);
    std::clog << "Ending program fk_spectrum; execution time (sec): " <<  duration.count()/1000000.0 << "\n";
    return 0;
}