#include <time.h>
#include <chrono>

#include <sys/stat.h>
#include <fstream>

#include <stdio.h>
#include <stdlib.h>
#include <string>
#include <math.h>

#include "axis.h"
#include "hypercube.h"
#include "floatHyperExt.h"
#include "float1DReg.h"
#include "float2DReg.h"
#include "functions.h"
#include "zeroPhaseFiltering2D.h"


using namespace SEP;


// ========================================
// ================ Main ==================
// ========================================


int main(int argc, char **argv) {
    
    std::clog << "Starting program generate_data \n";
	srand ( time(0) ); 
	auto start = std::chrono::high_resolution_clock::now();

// read in the list of input data files
    std::vector<std::string> input_data_list;
    for (int i=1; i<argc-1; i++){
        input_data_list.push_back(argv[i]);
    }

// read in the output data path
    std::string output_path;
    MB::readParam(argc, argv, "output_path", output_path, "./");

// initialize the vectors to store the input data files (including synthetic data and noise)
    std::vector<std::shared_ptr<float2DReg> > noise_list;
	std::shared_ptr<float2DReg> dat0 (new float2DReg(VECEXT::readHyper(input_data_list[0])));
    
    for (int i=1; i<input_data_list.size(); i++){
        std::shared_ptr<float2DReg> noise (new float2DReg(VECEXT::readHyper(input_data_list[i])));
        noise_list.push_back(noise);
    }

// read the data and noise values from binary files and store in the initialized vectors
    VECEXT::read(dat0, input_data_list[0]);
    for (int i=0; i<noise_list.size(); i++){
        VECEXT::read(noise_list[i], input_data_list[i+1]);
    }

// construct the low and high pass filter operators Fl & Fh
    axis X, Z;
    Z = dat0->getHyper()->getAxis(1);
    X = dat0->getHyper()->getAxis(2);

    std::shared_ptr<float1DReg> lpfilter = MB::sincWavelet(0.04,200,0.55);
	std::shared_ptr<float1DReg> lpfilter2 = MB::sincWavelet(0.08,200,0.55);
    std::shared_ptr<float1DReg> lpfilter3 = MB::sincWavelet(0.1,200,0.55);
    axis Zf = lpfilter->getHyper()->getAxis(1);
    axis Zf2 = lpfilter2->getHyper()->getAxis(1);
    axis Zf3 = lpfilter3->getHyper()->getAxis(1);
    Zf.d = Z.d;
    Zf2.d = Z.d;
    Zf3.d = Z.d;
    Zf.o = Z.o - Zf.d*(Zf.n/2);
    Zf2.o = Z.o - Zf2.d*(Zf2.n/2);
    Zf3.o = Z.o - Zf3.d*(Zf3.n/2);
    std::shared_ptr<hypercube> hyperf (new hypercube(Zf));
    std::shared_ptr<hypercube> hyperf2 (new hypercube(Zf2));
    std::shared_ptr<hypercube> hyperf3(new hypercube(Zf3));
    lpfilter->setHyper(hyperf);
	lpfilter2->setHyper(hyperf2);
    lpfilter3->setHyper(hyperf3);
	std::shared_ptr<float1DReg> hpfilter = lpfilter->clone();
	std::shared_ptr<float1DReg> hpfilter2 = lpfilter2->clone();
    std::shared_ptr<float1DReg> hpfilter3 = lpfilter3->clone();
    hpfilter->scale(-1);
	hpfilter2->scale(-1);
    hpfilter3->scale(-1);
    (*hpfilter->_mat)[200] += 1;
	(*hpfilter2->_mat)[200] += 1;
    (*hpfilter3->_mat)[200] += 1;
    
	zeroPhaseFiltering2D Fl4(lpfilter,dat0->getHyper(),dat0->getHyper());
    zeroPhaseFiltering2D Fl8(lpfilter2,dat0->getHyper(),dat0->getHyper());
    zeroPhaseFiltering2D Fl10(lpfilter3,dat0->getHyper(),dat0->getHyper());
    zeroPhaseFiltering2D Fh4(hpfilter,dat0->getHyper(),dat0->getHyper());
	zeroPhaseFiltering2D Fh8(hpfilter2,dat0->getHyper(),dat0->getHyper());
    zeroPhaseFiltering2D Fh10(hpfilter3,dat0->getHyper(),dat0->getHyper());


// construct the LF-HF data
	std::shared_ptr<float2DReg> datH4 = dat0->clone();	// High frequency data
    Fh4.forward(false,dat0,datH4);
    std::shared_ptr<float2DReg> datH410 = dat0->clone();
    Fl10.forward(false, datH4, datH410);
    std::shared_ptr<float2DReg> datH820 = dat0->clone();
    Fh8.forward(false,dat0,datH820);
    std::shared_ptr<float2DReg> datL4 = dat0->clone(); // Low frequency data
	Fl4.forward(false,dat0,datL4);

// add the noise to the data
    std::shared_ptr<float2DReg> datL_noisy (new float2DReg(dat0->getHyper()));
    std::shared_ptr<float2DReg> dat_noisy (new float2DReg(dat0->getHyper()));

    datL_noisy=datL4->clone();
    dat_noisy=dat0->clone();
    for (int i=0; i<noise_list.size(); i++){
        datL_noisy->add(noise_list[i]);
        dat_noisy->add(noise_list[i]);
    }

// write the data to disk
    VECEXT::write(datH4, output_path+"datH_4_20");
	VECEXT::write(datH410, output_path+"datH_4_10");
	VECEXT::write(datH820, output_path+"datH_8_20");
	VECEXT::write(datL4,output_path+"datL_2p5_4");
	VECEXT::write(datL_noisy,output_path+"datL_2p5_4_noisy");
    VECEXT::write(dat_noisy,output_path+"dat_2p5_20_noisy");

	
	auto stop = std::chrono::high_resolution_clock::now();
	auto duration = std::chrono::duration_cast<std::chrono::microseconds>(stop - start);
    std::clog << "Ending program generate_data; execution time (sec): " <<  duration.count()/1000000.0 << "\n";
    return 0;
}