//
//  main.cpp
//  PEF
//
//  Created by Milad Bader on 11/13/18.
//  Copyright © 2018 Milad Bader. All rights reserved.
//

#include "float2DReg.h"
#include "floatHyperExt.h"
#include "complexHyperExt.h"
#include "plotting.h"
#include "fkTransform.h"
#include "complex2DReg.h"

using namespace SEP;


// ========================================
// ================ Main ==================
// ========================================


int main(int argc, char **argv) {
    
    std::clog << "Hello World! \n" << std::endl;
	srand ( time(0) ); 

// ========================================
// ======== Reading all 2D data ===========
// ========================================

std::clog << "Reading data\n";

	axis X, Z, X2, Z2;
	X.o=0; // x origin (in m)
	Z.o=0; // z origin (in m)
	X.d=12.5; // x sampling (in m)
	Z.d=0.005; // (in sec)
	X.n=321; // number of samples in x
	Z.n=601; // number of samples in z
    X2 = X;
    Z2 = Z; Z2.n = Z.n /2 +1; // frequency axes for Fourier transforms

	std::shared_ptr<float2DReg> dat0 (new float2DReg(Z, X));
    std::shared_ptr<float2DReg> datH4 (new float2DReg(Z, X));
    std::shared_ptr<float2DReg> datH410 (new float2DReg(Z, X));
    std::shared_ptr<float2DReg> datH820 (new float2DReg(Z, X));
    std::shared_ptr<float2DReg> datL4 (new float2DReg(Z, X));
    std::shared_ptr<float2DReg> datL_noisy (new float2DReg(Z, X));
    std::shared_ptr<float2DReg> mod1 (new float2DReg(Z, X));
    std::shared_ptr<float2DReg> mod2 (new float2DReg(Z, X));
	
    VECEXT::read(dat0, "dat_init_2p5_20", "../data/seg_2019/");
	VECEXT::read(datH4, "datH_4_20", "../data/seg_2019/");
	VECEXT::read(datH410, "datH_4_10", "../data/seg_2019/");
	VECEXT::read(datH820, "datH_8_20", "../data/seg_2019/");
	VECEXT::read(datL4, "datL_2p5_4", "../data/seg_2019/");
	VECEXT::read(datL_noisy, "datL_2p5_4_noisy", "../data/seg_2019/");
    VECEXT::read(mod1, "mod_exp1_2p5_4", "../data/seg_2019/");
    VECEXT::read(mod2, "mod_exp2_2p5_4", "../data/seg_2019/");

	double rms = VECEXT::rms(dat0);
	double rmsH = VECEXT::rms(datH410);
	double rmsL = VECEXT::rms(datL4);
	double maxL = datL4->absMax();
    double rmsM1 = VECEXT::rms(mod1);
	double rmsM2 = VECEXT::rms(mod2);

// ========================================
// ======== Make and save figures =========
// ========================================

std::clog << "Output figures\n";

    /* {
    plot2D plt(dat0->getHyper());
    plt.setRange(-5*rms, 5*rms);
	plt.setTerminal("png");
	plt.setOutput("dat_init_2p5_20", "../fig/seg_2019/");
	plt.plot(dat0);
    }

    {
    plot2D plt(dat0->getHyper());
    plt.setRange(-5*rmsL, 5*rmsL);
	plt.setTerminal("png");
	plt.setOutput("datL_2p5_4_noisy", "../fig/seg_2019/");
	plt.plot(datL_noisy);
    }

    {
    plot2D plt(dat0->getHyper());
    plt.setRange(-5*rmsL, 5*rmsL);
	plt.setTerminal("png");
	plt.setOutput("datL_2p5_4", "../fig/seg_2019/");
	plt.plot(datL4);
    } */

    /* {
    plot2D plt(dat0->getHyper());
    plt.setRange(-5*rmsL, 5*rmsL);
	plt.setTerminal("pdf");
	plt.setOutput("mod_exp1_2p5_4", "../fig/seg_2019/");
	plt.plot(mod1);
    }

    {
    plot2D plt(dat0->getHyper());
    plt.setRange(-5*rmsL, 5*rmsL);
	plt.setTerminal("pdf");
	plt.setOutput("mod_exp2_2p5_4", "../fig/seg_2019/");
	plt.plot(mod2);
    } */

    std::shared_ptr<float2DReg> dat = datH4->clone();
    dat->add(datL_noisy);

    /* {
    plot2D plt(dat0->getHyper());
    plt.setRange(-5*rms, 5*rms);
	plt.setTerminal("pdf");
	plt.setOutput("dat_2p5_20_noisy", "../fig/seg_2019/");
	plt.plot(dat);
    }  */

    /* {
    fkTransform fk(dat0->getHyper());
    X2.d = fk.dk();
    X2.o = fk.ok();
    Z2.d = fk.df();
    Z2.o = 0;
    std::shared_ptr<complex2DReg> vec (new complex2DReg(Z2, X2));
    fk.forward(false,dat0,vec);
    std::shared_ptr<float2DReg> datfk (new float2DReg(vec->getHyper()));
    datfk = CVECEXT::module2(vec);
    plot2D plt(datfk->getHyper());
    plt.setXlabel("Wavenumber (1/m)");
    plt.setYlabel("Frequency (Hz)");
    plt.setXRange(-0.06, 0.0);
    plt.setYRange(0, 25, false);
    plt.setRange(0, 1e+06);
    plt.setTerminal("pdf");
    plt.setOutput("dat_init_fk_2p5_20", "../fig/seg_2019/");
    plt.plot(datfk);
    }

    {
    fkTransform fk(dat0->getHyper());
    X2.d = fk.dk();
    X2.o = fk.ok();
    Z2.d = fk.df();
    Z2.o = 0;
    std::shared_ptr<complex2DReg> vec (new complex2DReg(Z2, X2));
    fk.forward(false,dat,vec);
    std::shared_ptr<float2DReg> datfk (new float2DReg(vec->getHyper()));
    datfk = CVECEXT::module2(vec);
    plot2D plt(datfk->getHyper());
    plt.setXlabel("Wavenumber (1/m)");
    plt.setYlabel("Frequency (Hz)");
    plt.setXRange(-0.06, 0.0);
    plt.setYRange(0, 25, false);
    plt.setRange(0, 1e+06);
    plt.setTerminal("pdf");
    plt.setOutput("dat_fk_2p5_20_noisy", "../fig/seg_2019/");
    plt.plot(datfk);
    } */

    std::cerr << "\n 0 \n";
    return 0;
}