#include <time.h>
#include <chrono>

#include <sys/stat.h>
#include <fstream>

#include <stdio.h>
#include <stdlib.h>
#include <string>
#include <math.h>

#include "axis.h"
#include "floatHyperExt.h"
#include "float2DReg.h"
#include "functions.h"
#include "pef2D.h"
#include "cgls.h"
#include "convolution2D.h"


using namespace SEP;


// ========================================
// ================ Main ==================
// ========================================


int main(int argc, char **argv) {
    
    std::clog << "Starting program pef_lf_denoise_v1 \n";
	srand ( time(0) ); 
	auto start = std::chrono::high_resolution_clock::now();

// read in the input data
    std::shared_ptr<float2DReg> dat (new float2DReg(VECEXT::readHyper(argv[1])));
    VECEXT::read(dat, argv[1]);

// read in the 2D PEF
    std::string pef_file;
    MB::readParam(argc, argv, "pef", pef_file);
    std::shared_ptr<float2DReg> pef (new float2DReg(VECEXT::readHyper(pef_file)));
    VECEXT::read(pef, pef_file);
    unsigned int pef_nx = pef->getHyper()->getAxis(2).n;
    unsigned int pef_nz = pef->getHyper()->getAxis(1).n;

// read in other parameters
    int niter, pad_n1_start, pad_n2_start, pad_n1_end, pad_n2_end;
    float threshold;
    bool crossBounds;
    MB::readParam(argc, argv, "niter", niter, 100);
    MB::readParam(argc, argv, "threshold", threshold, 0.01);
    MB::readParam(argc, argv, "cross_bounds", crossBounds, false);
    MB::readParam(argc, argv, "pad_n1_start", pad_n1_start, 0);
    MB::readParam(argc, argv, "pad_n2_start", pad_n2_start, 0);
    MB::readParam(argc, argv, "pad_n1_end", pad_n1_end, 0);
    MB::readParam(argc, argv, "pad_n2_end", pad_n2_end, 0);

// read in the output file name
    std::string output;
    MB::readParam(argc, argv, "output", output, "model");

// estimate the model (de-noised data)
    std::shared_ptr<float2DReg> mod = VECEXT::pad(dat, pad_n2_start, pad_n2_end, pad_n1_start, pad_n1_end);
    axis X = mod->getHyper()->getAxis(2);
    axis Z = mod->getHyper()->getAxis(1);
    X.n = VECEXT::getnx(mod) + VECEXT::getnx(pef) - 1;
	Z.n = VECEXT::getnz(mod) + VECEXT::getnz(pef) - 1;
	X.o = VECEXT::getox(mod) + VECEXT::getox(pef);
	Z.o = VECEXT::getoz(mod) + VECEXT::getoz(pef);
    std::shared_ptr<float2DReg> zero (new float2DReg(Z,X)); // vector filled with zeros
    zero->zero();
    convolution2D conv(pef,mod->getHyper(),zero->getHyper()); // define the PEF convolution matrix
    conv.crossBounds(crossBounds);

    cgls cg(niter,threshold);
	cg.run(&conv, mod, zero);

    mod = VECEXT::cut(mod, pad_n2_start, pad_n2_end, pad_n1_start, pad_n1_end);

// write the model to disk
    VECEXT::write(mod, output);
	
	auto stop = std::chrono::high_resolution_clock::now();
	auto duration = std::chrono::duration_cast<std::chrono::microseconds>(stop - start);
    std::clog << "Ending program pef_lf_denoise_v1; execution time (sec): " <<  duration.count()/1000000.0 << "\n";
    return 0;
}