#include "Trace.h"
#include "functions.h"
#include <fftw3.h>



// Functions definition

// Print the values of a vector
void print(vector <TYPE> &v,string message, ostream &o){
    string espace(message.size(), ' ');
    o << message << " [" ;
    for (int j =0; j < v.size(); j++)
        o << " " << v[j];
    o << " ]" << endl;
}

// Print a complex number
void print(complex <TYPE> & z,ostream &o){
	string sign;
	if (z.imag()>0) o << z.real() << "+" << z.imag() << "i";
	else if (z.imag()<0) o << z.real() << z.imag() << "i";
	else o << z.real();
	o << endl;
}

// Plot a trace from ascii file using GNUPLOT
void plot(string file, string param1, string param2){
	// Read initial time and sampling interval from file
	int t0;
	TYPE si;
	ifstream in; in.open(file); cout << "Open failed: "<< in.fail() << endl;
	string line;
	stringstream linestream;
	getline(in,line);
	line.erase(0,4); linestream << line; linestream >> si; linestream.str("");linestream.clear();
	getline(in,line);
	line.erase(0,4); linestream << line; linestream >> t0; linestream.str("");linestream.clear();
	TYPE x0 = t0*si;
	// Plot the trace using GNUPLOT
	string title = file;
	FILE *f = popen("gnuplot -p","w");
	fprintf(f,"%s",param1.c_str());
	//fprintf(f,"set title \"%s\"\n", title.c_str());
	fprintf(f,"plot \"%s\" using (($0*%f)+%f):($1) ",file.c_str(),si,x0);
	fprintf(f," %s",param2.c_str());
	fprintf(f," notitle\n");
	fprintf(f,"exit\n");
	fflush(f);
	pclose(f);
}

// Save the plot of a trace from ascii to png file  using GNUPLOT
void saveplot(string file, string out, string term, string param1, string param2){
	// Read initial time and sampling interval from file
	int t0;
	TYPE si;
	ifstream in; in.open(file); //cout << "Open failed: "<< in.fail() << endl;
	string line;
	stringstream linestream;
	getline(in,line);
	line.erase(0,4); linestream << line; linestream >> si; linestream.str("");linestream.clear();
	getline(in,line);
	line.erase(0,4); linestream << line; linestream >> t0; linestream.str("");linestream.clear();
	TYPE x0 = t0*si;
	// Plot the trace using GNUPLOT
	string title = file;
	if (out == "") {
		out = file;
		out.erase(out.end()-4, out.end());
	}
	out = out + "." + term;
	FILE *f = popen("gnuplot -p","w");
	fprintf(f,"set terminal %s\n",term.c_str());
	fprintf(f,"set output \"%s\"\n", out.c_str());
	fprintf(f,"%s",param1.c_str());
	//fprintf(f,"set title \"%s\"\n", title.c_str());
	fprintf(f,"plot \"%s\" using (($0*%f)+%f):($1) ",file.c_str(),si,x0);
	fprintf(f," %s",param2.c_str());
	fprintf(f," notitle\n");
	fprintf(f,"set terminal x11\n");
	fprintf(f,"exit\n");
	fflush(f);
	pclose(f);
}

// Output discrete Fourier transform of data vector
vector <complex<TYPE> >* DFT(vector<TYPE> * vec){
	unsigned int N = vec->size();
	vector <complex<TYPE> > * dft = new vector <complex<TYPE> >(N);
	TYPE * pd = vec->data();
	complex<TYPE> * pf = dft->data();
	TYPE theta;
	for (int j=0; j<N; j++){
		for (int i=0; i<N; i++){
			theta = -2*M_PI*i*j/N;
			pf[j]=pf[j]+complex<TYPE>(pd[i]*cos(theta),pd[i]*sin(theta));
		}
	}
	return dft;
}

// Output discrete Fourier transform of data vector using FFTW3
vector <complex<TYPE> >* FFT(vector<TYPE> * vec){
	unsigned int N = vec->size();
	TYPE * in = vec->data();
	fftw_complex * out;
	out=(fftw_complex*)fftw_malloc(sizeof(fftw_complex)*(N/2+1));
	fftw_plan p;
	p = fftw_plan_dft_r2c_1d(N, in, out, FFTW_ESTIMATE);
	fftw_execute(p);
	complex <TYPE> * ptr;
	ptr = reinterpret_cast<std::complex<TYPE>* >(out);
	vector <complex<TYPE> >* fft = new vector <complex<TYPE> >(ptr, ptr+N/2+1);
	fftw_destroy_plan(p);
	fftw_free(out);
	return fft;
}

// Inverse Fourier transform
vector<TYPE>* inv_DFT(vector <complex<TYPE> >* dft){
	unsigned int N = dft->size();
	vector <TYPE> * v = new vector <TYPE>();
	complex<TYPE> * pf = dft->data();
    TYPE theta, sum=0;
    for (int j=0; j<N; j++){
        for (int i=0; i<N; i++){
            theta = 2*M_PI*i*j/N;
			complex<TYPE> comp(cos(theta), sin(theta));
            sum=sum + (pf[i] * comp).real();
        }
		sum=sum/N;
		v->push_back(sum);
		sum = 0;
    }
	return v;
}

// Inverse Fourier transform using FFTW3
vector<TYPE>* inv_FFT(vector <complex<TYPE> >*  fft){
	unsigned int N = 2*(fft->size()-1);
	if ((*fft)[fft->size()-1].imag()!=0.0) {N++;}
	fftw_complex * in = reinterpret_cast<fftw_complex* >(fft->data());
	TYPE * out;
	out=(TYPE*)fftw_malloc(sizeof(TYPE)*N);
    fftw_plan p;
	p = fftw_plan_dft_c2r_1d(N, in, out, FFTW_ESTIMATE);
	fftw_execute(p);
	vector <TYPE> * vec = new vector<TYPE>(out, out+N);
	fftw_destroy_plan(p);
	fftw_free(out);
	// Normalize the output by its size
	TYPE * ptr = vec->data();
	N = vec->size();
	for (int i=0; i<N; i++) ptr[i]=ptr[i]/N;
	return vec;
}


// Save real part of DFT into Trace
Trace* real(vector <complex<TYPE> >* dft, bool full_transform, TYPE SI, int t){
	unsigned int N = dft->size();
	Trace* tr = new Trace();
	tr->set_si(SI); tr->set_t0(t);
	vector <TYPE> * vec = new vector<TYPE>(N);
	complex<TYPE> * pf = dft->data();
	TYPE * pd = vec->data();
	if (full_transform){
		int up = ceil(N/2), dn = floor(N/2);
		for (int i=0; i<=dn; i++){pd[up+i-1]=pf[i].real();}
		for (int i=dn+1; i<N; i++){pd[i-dn-1]=pf[i].real();}
	}
	else {
		for (int i=0; i<N; i++) {pd[i]=pf[i].real();}
	}
	tr->set_vector(vec);
	return tr;
}

// Save imaginary part of DFT into Trace
Trace* imag(vector <complex<TYPE> >* dft, bool full_transform, TYPE SI, int t){
	unsigned int N = dft->size();
	Trace* tr = new Trace(); tr->set_si(SI); tr->set_t0(t);
	vector <TYPE> * vec = new vector<TYPE>(N);
	complex<TYPE> * pf = dft->data();
	TYPE * pd = vec->data();
	if (full_transform){
		int up = ceil(N/2), dn = floor(N/2);
		for (int i=0; i<=dn; i++){pd[up+i-1]=pf[i].imag();}
		for (int i=dn+1; i<N; i++){pd[i-dn-1]=pf[i].imag();}
	}
	else {
		for (int i=0; i<N; i++) {pd[i]=pf[i].imag();}
	}
	tr->set_vector(vec);
	return tr;
}

// Save modulus squared (amplitude spectrum or magnitude squared) of DFT into Trace
Trace* amp(vector <complex<TYPE> >* dft, bool full_transform, TYPE SI, int t){
	unsigned int N = dft->size();
	Trace* tr = new Trace(); tr->set_si(SI); tr->set_t0(t);
	vector <TYPE> * vec = new vector<TYPE>(N);
	complex<TYPE> * pf = dft->data();
	TYPE * pd = vec->data();
	if (full_transform){
		int up = ceil(N/2), dn = floor(N/2);
		for (int i=0; i<=dn; i++){pd[up+i-1]=norm(pf[i]);}
		for (int i=dn+1; i<N; i++){pd[i-dn-1]=norm(pf[i]);}
	}
	else {
		for (int i=0; i<N; i++) {pd[i]=norm(pf[i]);}
	}
	tr->set_vector(vec);
	return tr;
}

// Save argument (phase angle in radians) of DFT into Trace
Trace* phase(vector <complex<TYPE> >* dft, bool full_transform, TYPE SI, int t){
	unsigned int N = dft->size();
	Trace* tr = new Trace(); tr->set_si(SI); tr->set_t0(t);
	vector <TYPE> * vec = new vector<TYPE>(N);
	complex<TYPE> * pf = dft->data();
	TYPE * pd = vec->data();
	if (full_transform){
		int up = ceil(N/2), dn = floor(N/2);
		for (int i=0; i<=dn; i++){pd[up+i-1]=arg(pf[i]);}
		for (int i=dn+1; i<N; i++){pd[i-dn-1]=arg(pf[i]);}
	}
	else {
		for (int i=0; i<N; i++) {pd[i]=arg(pf[i]);}
	}
	tr->set_vector(vec);
	return tr;
}

// Define the 'sinc' function
TYPE sinc(const TYPE x){
	if (x==0.0) return 1.0;
    else return sin(x)/x;
}

// Build a cosine windowed sinc filter
// wc is the cutoff angular frequency (percentage of PI)
// N is the half length of the filter, in number of samples
// alpha is the coefficient used for cosine windowing
Trace* sinc_filter(TYPE wc, int N, TYPE alpha){
	vector <TYPE> * v = new vector<TYPE>();
	TYPE num;
	for (int i=0; i<2*N+1; i++){
		num = wc*sinc(wc*M_PI*(i-N))*(alpha +(1-alpha)*cos(M_PI*(i-N)/N));
		v->push_back(num);
	}
	Trace* tr = new Trace(v,1,-N);
	return tr;
}

// Build a centered Gaussian wavelet
// sig is the std deviation in number of samples
// N is the half length of the wavelet, in number of samples
Trace* gaussian(TYPE sig, int N){
	vector<TYPE> * v = new vector<TYPE>(2*N+1);
	TYPE num = 0;
	for (int i=N; i<2*N+1; i++){ 
		num = exp(-0.5*(i-N)*(i-N)/(sig*sig));
		(*v)[2*N-i]=num;
		(*v)[i]=num;
	}
	Trace* tr = new Trace(v,1,-N);
	return tr;
}

// Build a centered Ricker wavelet
// sig is the std deviation in number of samples
// N is the half length of the wavelet, in number of samples
// Central freq is wc = sqrt(2)/sig
Trace* ricker(TYPE sig, int N){
	vector<TYPE> * v = new vector<TYPE>(2*N+1);
	TYPE num = 0;
	for (int i=N; i<2*N+1; i++){ 
		num = (1-(i-N)*(i-N)/(sig*sig))*exp(-0.5*(i-N)*(i-N)/(sig*sig));
		(*v)[2*N-i]=num;
		(*v)[i]=num;
	}
	Trace* tr = new Trace(v,1,-N);
	return tr;
}

// Save complex vector to a 2 columns ascii file (real and imag)
void savec(vector<complex<TYPE> > * vec, string file){
	ofstream out; out.open(file);
	complex<TYPE> * p = vec->data();
	unsigned int N = vec->size();
	for (int i=0; i<N; i++) {out << p[i].real() <<"\t"<< p[i].imag() << endl;}
	out.close();
}

// Plot a complex vector from ascii file to complex plane using GNUPLOT
void plotc(string file, string param1, string param2){
	string title = file;
	FILE *f = popen("gnuplot -p","w");
	fprintf(f,"%s",param1.c_str());
	//fprintf(f,"set title \"%s\"\n", title.c_str());
	fprintf(f,"plot \"%s\" using ($1):($2) ",file.c_str());
	fprintf(f," %s",param2.c_str());
	fprintf(f," notitle\n");
	fprintf(f,"exit\n");
	fflush(f);
	pclose(f);
}

// Save the plot pf a complex vector from ascii file to complex plane using GNUPLOT
void saveplotc(string file, string out, string term, string param1, string param2){
	string title = file;
	if (out == "") {
		out = file;
		out.erase(out.end()-4, out.end());
	}
	out = out + "." + term;
	FILE *f = popen("gnuplot -p","w");
	fprintf(f,"set terminal %s\n",term.c_str());
	fprintf(f,"set output \"%s\"\n", out.c_str());
	fprintf(f,"%s",param1.c_str());
	//fprintf(f,"set title \"%s\"\n", title.c_str());
	fprintf(f,"plot \"%s\" using ($1):($2) ",file.c_str());
	fprintf(f," %s",param2.c_str());
	fprintf(f," notitle\n");
	fprintf(f,"exit\n");
	fflush(f);
	pclose(f);
}